import numpy as np
import scipy as sp

def convert_pure(sparse, single_sidelength, len_x):
    """
    Info:
    Args:
    Returns:
    """
    with_log = False

    sidelength = single_sidelength*len_x
    data = list()

    for i in range(len(sparse)):
        var = sparse[i]
        val1 = int(var)%sidelength
        val2 = int(int(var)/sidelength)
        val = (val1, val2)
        if with_log: 
            print("\n val: ", val)
        if val1 != val2 and val1 > val2: # TODO: Originally it was val1 < val2, why did 
                                         #    we now have to turn it around?
            #data[i][0] = val1
            #data[i][1] = val2
            data.append([val1, val2])
        else:
            pass
            #print("\n data i: ", i, "; equal val1: ", val1, "; val2: ", val2)
        if with_log: 
            print("\n data: ", data)

    data_dense = dense_matrix(data, sidelength)
    return data_dense

# TODO: Actually this function is copy-pasted from our other script - 
#     organize, that we don't write the same function twice
def dense_matrix(data, len_xy):
    """
    Info: 
    Args:
    Returns: 
    """
    
    with_log = False
    if with_log: 
        print("\n data_in: ", data)
    rows = [el[0] for el in data]
    cols = [el[1] for el in data]
    vals = [1 for el in data]

    dense_matrix = sp.sparse.coo_matrix((vals, (rows, cols)), shape=(len_xy, len_xy)).toarray()
    return dense_matrix




#import inspect, os

#print (inspect.getfile(inspect.currentframe())) # script filename (usually with path)
#print (os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))) # script directory

import os

print(os.path.realpath(__file__))



# try to look, when a common total cluster is created

import funcDictionary as dic
import funcDictionaryLevenshtein as dicLeven
import plotData

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import rcParams

import time

import sys

import imnet

import matplotlib.patches as mpatches
#from matplotlib.pypot import gca, show

from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
from matplotlib.colors import ListedColormap
from collections import OrderedDict

sys.path.append("./phasetransition/")
#import plot1 as p1
#import plot2 as p2
#import plot3 as p3
import plot4 as p4

import sys
import argparse

import graphics 
# todo: make an overall class plot

from mpl_toolkits.axes_grid1.inset_locator import (inset_axes, InsetPosition,
                                                  mark_inset)
# info: example command from the command line:
#     python performance.py --name="data/spar"

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--name", help="name under which the edge list is saved, usually: data/sparse_performance.txt")
    parser.add_argument("--idx_max", help="maximum index of a pathogen sequence, usually 0")
    parser.add_argument("--name_params", help="name under which to save the parameters, usually: data/sparse_params_performance.txt")
    parser.add_argument("--parser_N_part", help="number of sequences per single block, usually 4000 ")
    parser.add_argument("--parser_len_xy", help="number of single blocks in x and y direction respectively. \
            number of processed (not loaded) sequences: len_xy * N_part")
    
    args = parser.parse_args()
    name = args.name if not args.name == None else "data/sparse_performance.txt"
    idx_max = int(args.idx_max) if not args.idx_max == None else 0
    name_params = args.name_params if not args.name_params == None else "data/sparse_params_performance.txt"
    parser_N_part = int(args.parser_N_part) if not args.parser_N_part == None else 10**3
    parser_len_xy = int(args.parser_len_xy) if not args.parser_len_xy == None else 4
    
    plotData = plotData.PlotDataLevenshtein()
    
    graphics.design(2, 0.5)
    
    # info: set parameters
    plotData.clusterMinLen = 1
    plotData.N = 1*10**3
    plotData.min_ldVal = -1
    plotData.maxVal = 3
    plotData.max_ldVal = plotData.maxVal
    gpu_l = 8000 #5000
    step = 0
    
    #Ns = np.multiply([8, 16], 10**3)
    Ns = np.multiply([0.1, 0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 8.0], 10**3)
    #Ns = np.multiply([0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 4.0, 5.0], 10**3)
    
    #maxVals = [1, 2]
    maxVals = [1, 2, 3, 4, 5]
    
    xlabel = 0.02
    ylabel = 0.92
    
    DxLabel = 0.115
    DyLabel = 0.1
    
    # info: plotting the phase transition -> over N for fixed max_ldVal
    
    #fig, ax = plt.subplots(nrows = 1, ncols = 2) - info: two plots - only for masterthesis
    fig, ax = plt.subplots(nrows = 1, ncols = 2)
    
    #     but not in the paper
    #ax, ax_inset = p4.plot4(ax, ax_inset)
    ax[0], ax[1] = p4.plot4(ax[0], ax[1], name=name, idx_max=idx_max, name_params=name_params, parser_N_part=parser_N_part, parser_len_xy=parser_len_xy)
    
    #-ax[1][1] = p4.plot4(ax[1][1])
    
    print("\n /home/paul/Documents/imnet-master2/phasetransition.py: now going to show")
    plt.show()    
    
    fig.tight_layout()
    fig.savefig("performance.png", dpi=300, bbox_inches="tight")
    print("\n /home/paul/Documents/imnet-master2/phasetransition.py: saved")



# load net stuff:

import numpy as np
#import networkx as nx
import matplotlib.pyplot as plt
import time

from graph_tool.all import *
from . import graphics

from matplotlib import rcParams

import pandas as pd

import seaborn as sns
import numpy as np

import sys
import argparse

from . import pathogen_analysis as pa

def convert_pure(sparse, single_sidelength):
    """
    Info: 
    Args: 
    Returns: 
    """
    
    sidelength = single_sidelength*len_x
    data = None

    for i in range(len(sparse)):
        var = sparse[i]
        val1 = var%sidelength
        val2 = int(var/sidelength)
        val = (val1, val2)
        if val1 != val2 and val1 < val2:
            #data[i][0] = val1
            #data[i][1] = val2
            data.append([val1, val2])
        else:
            pass
            #print("\n data i: ", i, "; equal val1: ", val1, "; val2: ", val2)
    
    return data

def convert_edges(name="data/sparse.txt", len_x=10, single_sidelength=4*10**3):
    """
    Info: open file with name "name" and convert the contained single number indices
        to two-number indices
    Args: name: name of the file
        len_x: number of blocks in a single row
        single_sidelength: sidelength of a single block
    Returns: G: assembled graph
    """
    
    xs = list()
    ys = list()

    sidelength = single_sidelength*len_x
    data = None
    
    dic = {}

    with open(name, "r") as f:
        lines = f.readlines()
        #data = np.zeros([len(lines), 2])
        data = list()
        for i in range(len(lines)):
            #data = np.array([line.strip().split() for line in f],float)
            var = int(lines[i])

            #print("\n var: ", var, "; var.strip(): ", var.strip(), "; var.strip().split(): ", var.strip().split())
            #var = int(var.strip())
            val1 = var%sidelength
            val2 = int(var/sidelength)
            """
            val = (val1,val2)
            try:
                dic[val].append(1)
                print("\n --- i: ", i, "; vals twice")
            except:
                dic[val] = list()
            """
            #print("\n --- val1: ", val1, "; val2: ", val2, "; var: ", var, "; sidelength: ", sidelength)
            #data.append([val1, val2])
            if val1 != val2 and val1 < val2:
                #data[i][0] = val1
                #data[i][1] = val2
                data.append([val1, val2])
            else:
                pass
                #print("\n data i: ", i, "; equal val1: ", val1, "; val2: ", val2)
    #print("\n L: ", len(lines))
    #lines = np.loadtxt('sparse.txt')
    return data

def assemble_graph(data, cut_value = -1):
    """
    Info: assemble graph from two-number edge list "data"
    Args: data: two-number edgelist
        cut_value: int number: In case you only want to use the first 
            N edges of the file, you can set input some cut_value=N, otherwise 
            it should usually be set to -1.
    Returns: G: assembled graph
    """

    # info: make a graph and 
    G = Graph()
    ug = Graph(directed = False)
    # info: adding nodes, in order to first create all the nodes for the network
    t0 = time.time()
    #G.add_nodes_from(list(range(len(data))))
    #for i )):
    t1 = time.time()
    
    # info: adding the edges
    #G.add_edges_from(data)
    vals = list()
    dic = {}
    len_data = len(data)
    vals_edge_list = list()
   
    # TODO: D the analysis for higher index
    if cut_value == -1:
        len_val = len(data)
    else:
        len_val = min(len(data), cut_value)
    
    data_1 = [el[0] for el in data]
    data_2 = [el[1] for el in data]
    max_val = max([max(data_1), max(data_2)])
    print("\n max_val: ", max_val, "; len_val: ", len_val)
    G.add_vertex(max_val)#(len_val)
    #print("\n A G.vertices: ", len(list(G.vertices())))
    for i in range(len_val): 
        vals_edge_list.append([data[i][0], data[i][1]])
        if i%10000==0:
            print("\n i: ", i, " / ", len_val)
    G.add_edge_list(vals_edge_list)
    
    """
    for i in range(len_val):#(len(data)):#range(len(data)):
        data_i0 = data[i][0]
        data_i1 = data[i][1]
        
        try: 
            idx_1 = dic[data_i0]
        except: 
            idx_1 = len(list(dic.keys()))
            dic[data_i0] = idx_1

        try:
            idx_2 = dic[data_i1]
            vals_edge_list.append([idx_2])
        except:
            idx_2 = len(list(dic.keys()))
            dic[data_i1] = idx_2

        vals_edge_list.append([idx_1, idx_2])
        ""
        if data_i0 in vals: 
            pass
        else: 
            vals.append(data_i0)
        
        if data_i1 in vals:
            pass
        else:
            vals.append(data_i1)
        vals_edge_list.append([vals.index(data_i0), vals.index(data_i1)])
        ""
        #G.add_edge(vals.index(data_i0), vals.index(data_i1))
        
        #G.add_edge(data[i][0], data[i][1])
        if i%1000 == 0:
            print("\n i: ", i, " / ", len_data, "; data[i][0]: ", data[i][0], "; data[i][1]: ", data[i][1], "; len(list(G.vertices())): ",  len(list(G.vertices())), "; dt: ", (time.time() - t1))
        if i%10**4 == 0:
            print("\n dt: ", (time.time() - t1))
    """
    #print("\n len(vals_edge_list): ", len(vals_edge_list))
    t2 = time.time()    
    for v in G.vertices():
        neighbors = v.all_neighbors()
        #print("\n v: ", [G.vertex_index[neighbor] for neighbor in neighbors])
    #print("\n len(data): ", len(data)) 
    #print("\n len(list(G.vertices())): ",  len(list(G.vertices())))
    """   
    # info: if the graph is empty, then add at least one node,
    #     which is called the "ZeroNode"
    if len(G.nodes()) == 0:
        G = nx.Graph()
        G.add_node("ZeroNode")
    """
    return G



import numpy as np 
import scipy.special as spec
import matplotlib.pyplot as plt

def barabasi_p_degree(k, l, m):
    """
    info: k and l must always be equal or bigger to m
    input: 
    output: 
    """
    if m > min(k, l):
        print("\n /home/paul/Documents/imnet-master2: WARNING: m > max(k, l)")
    c1 = m*(k+2)/(k*l*(l+1))
    c2 = 1 - (spec.binom(2*m+1, m+1)*spec.binom(k+l-2*m, l-m))/spec.binom(k+l+2, l+1)

    return c1*c2

def barabasi_mean_degree(n, m, deg_0):
    """
    info: 
    input: 
    output: 
    """
    mean_deg = 0

    for deg in range(m, n): 
        p_deg = barabasi_p_degree(deg_0, deg, m)
        mean_deg += deg*p_deg

    return mean_deg

def barabasi_dist(n, m):
    """
    info: 
    input: 
    output: 
    """
    deg = np.zeros(n)

    for deg_0 in range(m, n):
        deg[deg_0] = barabasi_mean_degree(n, m, deg_0)

    return deg

def barabasi_plot():
    n = 20
    m = 1
    plt.plot(barabasi_dist(n, m))
    plt.show()

if __name__ == "__main__":
    barabasi_p__degree(1, 2, 1)



# INFO: convert indices: 

import numpy as np
from . import make_net 

import argparse
import sys
import scipy as sp

def dense_matrix(data, len_xy):
    """
    Info: 
    Args:
    Returns: 
    """
    
    print("\n data_in: ", data)
    rows = [el[0] for el in data]
    cols = [el[1] for el in data]
    vals = [1 for el in data]
    
    dense_matrix = sp.sparse.coo_matrix((vals, (rows, cols)), shape=(len_xy, len_xy)).toarray()
    return dense_matrix

if __name__ == "__main__": 
    parser = argparse.ArgumentParser()
    parser.add_argument('--name', help="name of the single-number-index file, default data/sparse.txt")
    parser.add_argument('--single_sidelength', help="number of sequences per single block, \
            default 4*10**3, better don't change that")
    parser.add_argument('--len_xy', help="number of single blocks in x and y direction respectively. \
            number of processed (not loaded) sequences: len_xy * N_part")
    parser.add_argument('--target_name', help="target name, under which the new list is saved")
    parser.add_argument('--format', help="sparse: makes the two-number-indices list; \
            dense: makes the entire matrix in dense format - don't use that for very \
            large data, since it consumes high amounts of memory")
    args = parser.parse_args()
    
    name = args.name if not args.name == None else "data/sparse.txt"
    len_x = int(args.len_xy) if not args.len_xy == None else 10
    single_sidelength = int(args.single_sidelength) if not args.single_sidelength == None else 4*10**3
    target_name = args.target_name if not args.target_name == None else "two_number_indices.txt"
    is_dense = True if args.format == "dense" else False
    
    data = make_net.convert_edges(name=name, len_x=len_x, single_sidelength=single_sidelength)
    print("\n data A: ", data)
    if is_dense:
        data = dense_matrix(data, len_x*single_sidelength)

    with open(target_name, "w") as f: 
        np.savetxt(f, data, fmt='%i')
    print("\n saved under \"", target_name, "\".")




class PlotData:
    def __init__(self):
        self.xArgsName = ""
        self.clustersizesName = ""
        self.clustersizesAllErrName = ""
        self.diametersName = ""
        self.diametersAllErrName = ""
        self.eccentritiesName = ""
        self.degreeMeanName = ""
        self.degreeMeanAllErrName = ""
        
        self.xArgs = list()
        self.clustersizes = list()
        self.clustersizesAllErr = list()
        self.diameters = list()
        self.diametersAllErr = list()
        self.eccentrities = list()
        self.degreeMean = list()
        self.degreeMeanAllErr = list()
 
class PlotDataSingleNet(PlotData):
    def __init__(self):
        self.cPre = list()
        self.cPost = list()
        self.clustersizesPre = list()
        self.clustersizesPost = list()
        self.GnamesHist1Pre = list()
        self.GnamesHist1Post = list()
        
        self.N = -1
        self.min_ldVal = -1
        self.max_ldVal = -1
        self.extractNum = -1
    
class PlotDataManyNets():
    def __init__(self):
        self.clustersizesMaxPre = list()
        self.clustersizesMaxPost = list()
        self.degMaxsPre = list()
        self.degMaxsPost = list()
        self.Ns = list()
        self.isConnectedsPre = list()
        self.isConnectedsPost = list()
        self.clustersizesMaxPost = list()
       
        self.maxValIndices = []
 
        self.Ns = []
        #info: number of extracted letters from the DNA sequence 
        self.min_ldVal = -1
        self.max_ldVal = -1
        self.extractNum = -1

        # info: samples = 6 is usual
        self.samples = -1

        # info: maximum levenshtein distance
        self.maxValIndices = []

class PlotDataLevenshtein: 
    def __init__(self):
        self.xArgsName = ""
        self.clustersizesName = ""
        self.clustersizesAllErrName = ""
        self.diametersName = ""
        self.diametersAllErrName = ""
        self.eccentritiesName = ""
        self.degreeMeanName = ""
        self.degreeMeanAllErrName = ""

        self.xArgs = list()
        self.clustersizes = list()
        self.clustersizesAllErr = list()
        self.diameters = list()
        self.diametersAllErr = list()
        self.eccentrities = list()
        self.degreeMean = list()
        self.degreeMeanAllErr = list()
  
        self.clustersizesAllArray = list()
        self.diametersAllArray = list()
        self.centersAllArray = list()
        self.eccentritiesAllArray = list()
        self.degreeMeanAllArray = list()
        self.lengthsAllArray = list()

        # info: only needed for lenNet2.py: 
        self.xArgsAllArray = list()

class InnerDataLevenshtein:
     def __init__(self):
         self.clustersizes = list()
         self.diameters = list()
         self.centers = list()
         self.eccentrities = list()
         self.degreeMean = list()
         
         # info: only needded for lenNet2:
         self.clustersizesAllArray = list()
         self.diametersAllArray = list()
         self.centersAllArray = list()
         self.eccentritiesAllArray = list()
         self.degreeMeanAllArray = list()
         self.xArgsAllArray = list()

         self.lengths = list()

class InnerDataManyNets: # I think, we should find away to reduce the number of members here
     def __init__(self):
         self.eccs = list()
         self.dias = list()
         self.degMaxs = list()
         self.nodes = list()
         self.cons = list()
         self.clustersizes = list()
         self.clustersizesMax = list()
         self.isConnected = list()
         
         self.eccsPre = list()
         self.diasPre = list()
         self.degMaxsPre = list()
         self.nodesPre = list()
         self.consPre = list()
         self.clustersizesPre = list()
         self.clustersizesMaxPre = list()
         self.isConnectedsPre = list()

         self.eccsPost = list()
         self.diasPost = list()
         self.degMaxsPost = list()
         self.nodesPost = list()
         self.consPost = list()
         self.clustersizesPost = list()
         self.clustersizesMaxPost = list()
         self.isConnectedsPost = list()

class DataObject:
    def __init__(self):
        self.clustersizes = list()
        self.diameters = list()
        self.centers = list()
        self.eccentricities = list()

        self.clustersizesAllArray = list()
        self.diametersAllArray = list()
        self.centersAllArray = list()
        self.eccentricitiesAllArray = list()
        self.degreeMeanAllArray = list()
        self.xArgsAllArray = list()



# info: plot3.py

import funcDictionary as dic
#import funcDictionaryLevenshtein as dicLeven

import networkx as nx
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import random
#import funcDictionaryLevenshtein as dicLeven
import plotData as pD

import pandas as pd

import matplotlib.patches as mpatches
# from matplotlib.pypot import gca, show

from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
from matplotlib.colors import ListedColormap
from collections import OrderedDict

#import graph_tool as gt
#from graph_tool.all import *
def degree_mean(g):
    deg = dict(g.degree())
    deg_values = list(deg.values())
    deg_mean = np.mean(deg_values)
    return deg_mean
def public_data(thresholds, dic_num_values): 
    """
    info: sort the values of the dictionary into the thresholds
    input: thresholds: an array of the thresholds (usually e.g. [1, 2, 3, 4, 5, 6, 7])
        dic_num_values: ...
    output: public_nums_normed: ...
    """
    print("\n --- thresholds: ", thresholds)
    print("\n --- len(dic_num_values): ", len(dic_num_values))

    public_nums = np.zeros(len(thresholds))
    for j in range(len(thresholds)):
        threshold = thresholds[j]
        public_num = 0
        for i in range(len(dic_num_values)):
            if dic_num_values[i] > threshold:
                public_num += 1
        public_nums[j] = public_num

    for i in range(len(thresholds)):
        pass

    len_dic_num_values = len(dic_num_values)
    public_nums_normed = [el/len_dic_num_values for el in public_nums]


    return public_nums_normed

def frequency_distribution(seq, samples, sample_size, thresholds):
    """
    info: make frequency distribution of the sequences
    input: seq: list of sequences
        samples: number of samples
        sample_size: size of a sample
        thresholds: list of thresholds, usually 
            is [1, 2, 3, 4, 5, 6, 7]
    output: public_nums_normed: the finished data of the frequencies over the threshold values
    """
     
    print("\n AAA len(seq): ", len(seq))
    dic = {}
    dic_num = {}

    # info: make normal list, which
    #     includes the indices, 
    #     where each sequence occurs
    #     (so each element is a list)
    for i in range(len(seq)):
        try: 
            dic[seq[i]].append(i)
        except: 
            dic[seq[i]] = list()
            dic[seq[i]].append(i)
   
    dic_values = list(dic.values())

    for i in range(len(dic_values)):
        for j in range(len(dic_values[i])):
            for sample in range(samples):
                lower_bound = sample*sample_size
                upper_bound = (sample+1)*sample_size

                if dic_values[i][j] >= lower_bound:
                    if dic_values[i][j] < upper_bound:
                        try: 
                            dic_num[seq[i]] += 1
                        except: 
                            dic_num[seq[i]] = 1
    
    
    dic_num_values = list(dic_num.values())
    
    public_nums_normed = public_data(thresholds, dic_num_values)

    return public_nums_normed, dic_num

def load_seq_choices(i, N):
    """
    Info: 
    Args: 
    Returns: 
    """
    print("\n --- load_seq_choices i: ", i, "N: ", N)
    src_name = "sequences/sequence_" + str(i+1) + ".txt"#name
    
    plotData = pD.PlotDataLevenshtein()
    plotData.clusterMinLen = 1
    plotData.N = N#10**6#4*10**3
    plotData.min_ldVal = -1
    plotData.maxVal = 3
    plotData.max_ldVal = plotData.maxVal
    gpu_l = 8000 #5000
    step = 0
    _, _, seq, filename, _ = dic.loadSequence(step, plotData, isExtractNum=False, src=src_name)
    
    return seq

def find_public(seq, sample_size, ax_10, ax_public_simple, name, samples_big):
    """
    info: find the public sequences among a list 
        of sequences
    input: seq: set of the sequences
        sample_size: size of a sample
    output: 
    """
    
    print("\n A -- A -- A : len(seq): ", len(seq))

    # info: set label parameters
    xlabel = 0.02
    ylabel = 0.92

    DxLabel = 0.115
    DyLabel = 0.1

    samples = int(len(seq)/sample_size)
    print("\n --- samples: ", samples, "; len(seq): ", len(seq))

    #dic_num_mean = frequency_distribution(seq, samples, sample_size)
    
    # info: find out, how many public exist:
    thresholds = [0, 1, 2, 3, 4, 5, 6]
    public_nums_array = []

    #samples_big = 2#20
    for i in range(samples_big):
        # info: randomly choose a subsample, in order to make many samples
        #seq_choices = random.sample(seq, int(len(seq)/2))
        seq_choices = load_seq_choices(i, len(seq))

        if i == 0:
            public_nums_normed, dic_num = frequency_distribution(seq_choices, samples, sample_size, thresholds)
            # info: plot data points
            #-ax_10.plot(thresholds, public_nums_normed, '.', color='blue')
            public_nums_array.append(public_nums_normed)
            public_nums_normed = [el/samples_big for el in public_nums_normed]
        else:
            public_nums_normed_i, _ = frequency_distribution(seq_choices, samples, sample_size, thresholds)
            # info: plot data points
            #-ax_10.plot(thresholds, public_nums_normed_i, '.', color='blue')
            public_nums_array.append(public_nums_normed_i)
            public_nums_normed = [public_nums_normed[j] + public_nums_normed_i[j]/samples_big for j in range(len(public_nums_normed))]
    

    # info: We give these empty start data, just
    #     to shift the seaborn plot, thus it is the 
    #     pair (-1, 0) is just for graphics design
    data_x = [-1]
    data_y = [0]
    data_x_reduced = []
    data_y_reduced = []
    
    for i in range(len(public_nums_array)):
        for j in range(len(public_nums_array[i])):
            data_x.append(j)
            data_y.append(public_nums_array[i][j])
            # TODO: We now plot only j=2; Is that already accumulated or should
            #     we still do that?
            if j == 1:
                data_x_reduced.append(j)
                data_y_reduced.append(public_nums_array[i][j])
    
    y_mean = np.mean(data_y_reduced)
    y_std = np.std(data_y_reduced)
    #TODO: std okay or should we make /sqrt(N)
    
    """
    print("\n --- data_x: ", data_x)
    print("\n --- data_y: ", data_y)
    print("\n data_x_reduced: ", data_x_reduced)
    print("\n data_y_reduced: ", data_y_reduced)
    print("\n y_mean: ", y_mean)
    print("\n y_std: ", y_std)
    """
    data = pd.DataFrame({"x": data_x, "y": data_y})
    data_reduced = pd.DataFrame({"x": data_x_reduced, "y": data_y_reduced})

    #sns.stripplot(x="x", y="y", data=data_reduced, ax=ax_public_simple)
    ax_public_simple.bar([0], [public_nums_normed[1]], zorder=-1.0)
    #ax_public_simple.errorbar([0], [y_mean], yerr=[y_std], capsize=100.0, zorder=1.0)
    sns.violinplot(x="x", y="y", data=data_reduced, ax=ax_public_simple, color=(0.6, 0.6, 1.0, 0.5), linewidth=0, zorder = 3, width=0.5)

    ax_public_simple.set_xlim(-0.5, 0.5)
    ax_public_simple.set_ylim(bottom=0.0)
    ax_public_simple.set_xticks([])
    ax_public_simple.set_xticklabels([])
    ax_public_simple.set_ylabel("$f_{k=2}$")
    ax_public_simple.set_xlabel("public")
    
    thresholds = [el+1 for el in thresholds]
    if name=="human":
        col="green"#"orange"
    if name=="mouse":
        col=(0.3, 0.3, 0.8, 1.0)
    ax_10.plot(thresholds, public_nums_normed, zorder=-1, label=name, color=col)
    sns.violinplot(x="x", y="y", data=data, ax=ax_10, linewidth=0.0, color=(0.6, 0.6, 1.0, 0.5))
    ax_10.set_xticks([1, 2, 3, 4, 5, 6, 7])
    ax_10.set_xticklabels(["1", "2", "3", "4", "5", "6", "7"])
    ax_10.set_xlim(1, max(thresholds))
    ax_10.set_ylim(10**(-4), 10**0)
    #ax_10.set_ylim(0, max(public_nums))
    ax_10.set_yscale('log')
    ax_10.set_xlabel("$k$")
    ax_10.set_ylabel("$f_k(k)$")
    ax_10.legend(frameon=False)

    ax_10.add_patch(Polygon([[0.0, 1.0], [DxLabel, 1.0], [DxLabel, 1.0 - DyLabel], [0.0, 1.0 - DyLabel]],
      closed=False, fill=True, color="white", alpha=0.9, transform = ax_10.transAxes, zorder=2))
    ax_10.text(xlabel, ylabel, '(a)', transform = ax_10.transAxes, zorder=3)

    return dic_num, ax_10, ax_public_simple

def network_properties(g_vdj, seqs_original, ax_10, ax_public_simple, name, samples_big, with_graph=True):
    # Why is he using g_vdj AND seqs_original?
    """
    info: %...
    input: %...
    output: %...
    """
    
    dic_num, ax_10, ax_public_simple = find_public(seqs_original, int(len(seqs_original)/10), ax_10, ax_public_simple, name, samples_big)
    with_graph = False
    if with_graph: 
        g_public = nx.Graph()
        g_vdj_nodes = list(g_vdj.nodes())

        threshold = 2

        print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: len(dic_num): ", len(dic_num))

        for i in range(len(g_vdj_nodes)):
            try:
                if i%1000==0:
                    print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: i: ", i)
                if dic_num[seqs_original[i]] > threshold:
                    g_public.add_node(i)
            except:
                print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: i: ", i, "; WARNING: there seems to be an element, that is not present in collection. (paul)")
        
        edges_list = list(g_vdj.edges())

        for i in range(len(edges_list)):
            if (edges_list[i][0] in g_public) and (edges_list[i][1] in g_public):
                g_public.add_edge(edges_list[i][0], edges_list[i][1])
                #gt_public.add_edge(edges_list[i][0], edges_list[i][1])

        # info: calculate_properties: 
        degree_mean_vdj = degree_mean(g_vdj)
        degree_mean_public = degree_mean(g_public)
        """-
        assortativity_coefficient_vdj = nx.degree_assortativity_coefficient(g_vdj)
        if len(list(g_public.nodes())) > 0:
            assortativity_coefficient__public = nx.degree_assortativity_coefficient(g_public)
        else: 
            assortativity_coefficient__public = -1
            print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: The g_public is 0, you should have a look.")
        """
        assortativity_coefficient_vdj = -1
        assortativity_coefficient__public = -1
    else: 
        degree_mean_vdj = -1
        degree_mean_public = -1
        assortativity_coefficient_vdj = -1
        assortativity_coefficient__public = -1

    return degree_mean_vdj, degree_mean_public, assortativity_coefficient_vdj, assortativity_coefficient__public, ax_10, ax_public_simple

def plot3(ax_10, g_vdj, seqs_originals, ax_a, ax_public_simple, samples_big=2):
    """
    info: 
    input: 
    output: 
    """
    
    names = ["mouse", "human"]
    print("\n CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC")

    for name in names: 
        ax_10, ax_a, ax_public_simple, ax_b = plot3_species(ax_10, g_vdj, seqs_originals[name], ax_a, ax_public_simple, name, samples_big)

    return ax_10, ax_a, ax_public_simple, ax_b

def plot3_species(ax_10, g_vdj, seqs_original, ax_a, ax_public_simple, name, samples_big):
    """
    info: %...
    input: %...
    output: %...
    """ 
    
    #g_vdj = dicLeven.make_graph(seq_sample, min_ld=plotData.min_ldVal,
    #            max_ld=plotData.max_ldVal, gpu_l=gpu_l)

    # info: make samples, where for each sample a network is created 
    #     and the network properties are calculated
    samples = 1
    
    degree_mean_vdj_array = []
    degree_mean_public_array = []
    assortativity_coefficient_array = []
    assortativity_coefficient_public_array = []

    for i in range(samples):
        print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: ", \
                " samples: ", i, " / ", samples)
        #-g_vdj = dicLeven.make_graph(seqs_original, min_ld=-1, \
        #-       max_ld=2, gpu_l=8000)
        g_vdj = None

        # info: loading the seqs_ax_10 sequences
        plotData = pD.PlotDataLevenshtein()
        plotData.clusterMinLen = 1
        plotData.N = len(seqs_original)#-10**6
        plotData.min_ldVal = -1
        plotData.maxVal = 3
        plotData.max_ldVal = plotData.maxVal
        a, a4, seqs_ax_10, filename, ls = dic.loadSequence(0, plotData, isExtractNum=False, animal=name)

        print("\n len(seqs_ax_10): ", len(seqs_ax_10))

        # info: proceed with the network properties
        degree_mean_vdj, degree_mean_public, assortativity_coefficient_vdj, \
            assortativity_coefficient_public, ax_10, ax_public_simple = network_properties(g_vdj, seqs_ax_10, ax_10, ax_public_simple, name, samples_big)

        degree_mean_vdj_array.append(degree_mean_vdj)
        degree_mean_public_array.append(degree_mean_public)
        assortativity_coefficient_array.append(assortativity_coefficient_vdj)
        assortativity_coefficient_public_array.append(assortativity_coefficient_public)
        
    ones = list(np.ones(len(degree_mean_vdj_array)))
    twos = [el*2 for el in ones]
    
    xs = ones + ones + twos + twos
    ys = degree_mean_vdj_array + degree_mean_public_array + assortativity_coefficient_array + assortativity_coefficient_public_array
    h = ones + twos + ones + twos
    total_array = pd.DataFrame({"x": ys, "y": xs, "hue": h})

    degree_mean_vdj_array = pd.DataFrame({'x':
        ones, 'y':
        degree_mean_vdj_array})
    degree_mean_public_array = pd.DataFrame({'x':
        twos,
        'y': degree_mean_public_array})
    assortativity_coefficient_array = pd.DataFrame({'x':
        ones,
        'y': assortativity_coefficient_array})
    assortativity_coefficient_public_array = pd.DataFrame({'x': 
        twos, 
        'y': assortativity_coefficient_public_array})
    
    """
    print("\n /home/paul/Documents/imnet-master2/public_analysis/plot3.py: ", \
            "degree_mean_vdj_array: ", degree_mean_vdj_array)
    print("\n degree_mean_public_array: ", degree_mean_public_array)
    print("\n assortativity_coefficient_array: ", assortativity_coefficient_array)
    print("\n assortativity_coefficient_public_array: ", assortativity_coefficient_public_array)
    
    print("\n degree_mean_vdj: ", degree_mean_vdj)
    print("\n degree_mean_public: ", degree_mean_public)
    print("\n assortativity_coefficient_vdj: ", assortativity_coefficient_vdj)
    print("\n assortativity_coefficient_public: ", assortativity_coefficient_public)
    ""
    sns.stripplot(x="y", y="x", data=degree_mean_vdj_array, ax=ax_a, orient='h', color='red')
    sns.stripplot(x="y", y="x", data=degree_mean_public_array, ax=ax_a, orient='h', color='red')
    sns.stripplot(x="y", y="x", data=assortativity_coefficient_array, ax=ax_a, orient='h', color='red')
    sns.stripplot(x="y", y="x", data=assortativity_coefficient_public_array, ax=ax_a, orient='h', color='red')
   
    print("\n total_array: ", total_array)
    sns.violinplot(x="x", y="y", hue="hue", data=total_array, ax=ax_a, orient='h', width=0.7, color=(0.5, 0.8, 1.0, 0.8))
    #sns.violinplot(x="x", y="y", hue="hue", data=data, ax=ax1, width=0.7, color=(0.5, 0.8, 1.0, 0.8))
    """
    width = 0.6
    #-col_blue = (0.5, 0.5, 1.0, 1.0)
    #-col_red = (1.0, 0.5, 0.5, 1.0)
    if name == "mouse": 
        col_blue = (0.5, 0.5, 1.0, 0.5)
        col_red = (1.0, 0.5, 0.5, 0.5)
    if name == "human":
        col_blue = (0.5, 0.5, 1.0, 0.5)
        col_red = (1.0, 0.5, 0.5, 0.5)

    ax_a.barh([0.0], [degree_mean_vdj], color=col_blue, zorder=-3, height=width)
    ax_a.barh([0.5], [degree_mean_public], color=col_red, zorder=-2, height=width)
    #sns.stripplot(x="x", y="y", data=test_data, ax=ax_a)
    """
    ax_a.bar(g = sns.catplot(
        data=penguins, kind="bar",
        x="species", y="body_mass_g", hue="sex",
        ci="sd", palette="dark", alpha=.6, height=6
        ))
    ax_b.bar()
    """
    """
    ax_a.set_ylabel("degree")
    ax_b.set_ylabel("DAC")
    ax_a.set_xticks([1, 2])
    ax_b.set_xticks([1, 2])
    ax_a.set_xticklabels(["public", "private"])
    ax_b.set_xticklabels(["public", "private"])
    """
    ax_a.set_yticks([0.25, 1.75])
    ax_a.set_yticklabels(["$d$", "$a$"])
    #-ax_a.legend(["total", "public"], frameon = False)
    ax_a.legend(["total", "public"], frameon = False, loc="lower right")
    ax_a.set_xlabel("degree $d$")
    ax_a.set_xlim([0.0, 30.0])

    ax_b = ax_a.twiny()

    ax_b.barh([1.5], [assortativity_coefficient_vdj], color=col_blue, zorder=-3, height=width)
    ax_b.barh([2.0], [assortativity_coefficient_public], color=col_red, zorder=-2, height=width)
    ax_b.set_xlim([0.0, 1.0])
    ax_b.set_xlabel("assortativity $a$")
    
    #plt.setp(ax_b.get_yticklabels(), rotation=45)
    
    #ax_b = ax_a.twinx()
    #ax_b = ax_a.twinx(ii)

    #ax_b.set_yticklabels(["", ""])
    #ax_b.legend([], frameon = False)
    #ax_b.

    """
    g_private = nx.Graph()
    g_public = nx.Graph()
    #gt_public = gt.Graph()

    collection_keys = collection.keys()
    collection_values = collection.values()
     
    seqs = list(g_vdj.nodes())

    # info: suppose, that the order of sequences
    #     in collection is the same as in seqs

    for i in range(len(seqs)):
        try: 
            if collection[seqs_original[i]] > 2:
                g_public.add_node(i)
                #gt_public.add_vertex(i)
        except:
            print("\n WARNING: there seems to be an element, that is not present in collection. (paul)")

    edges_list = list(g_vdj.edges())
 
    for i in range(len(edges_list)):
        if (edges_list[i][0] in g_public) and (edges_list[i][1] in g_public):
            g_public.add_edge(edges_list[i][0], edges_list[i][1])
            #gt_public.add_edge(edges_list[i][0], edges_list[i][1])
    
    nodes_public = len(list(g_public.nodes()))
    nodes_total = len(list(g_vdj.nodes()))
    
    print("\n nodes_public: ", nodes_public)
    print("\n nodes_total: ", nodes_total)
    print("\n nodes_public/nodes_total: ", nodes_public/nodes_total)
    """
    return ax_10, ax_a, ax_public_simple, ax_b



import time
import matplotlib.pyplot as plt
import numpy as np

#from numba import cuda

#import funcDictionary as dic
#import funcDictionaryLevenshtein as dicLeven
#import plotData
#import textdistance as td

#from scipy import sparse
import pandas as pd
import math

#from numbapro import vectorize

def specificity():
    """
    info: 
    input: 
    output: 
    """

    df = pd.read_excel('specific.xlsx')
    specs_raw = list(df['BetaConfi'])
    CDR3s_raw = list(df['CDR3beta'])
    print("\n len(specs_raw): ", len(specs_raw), "; len(CDR3s_raw): ", len(specs_raw))
    specs = list()
    CDR3s = list()
    for i in range(len(specs_raw)):
        if math.isnan(specs_raw[i]) == False and math.isnan(specs_raw[i]) == False:
            specs.append(float(specs_raw[i]))
            CDR3s.append(CDR3s_raw[i])
        #data = pd.read_csv('virus_databank.csv', low_memory=False)

    idx_max = len(CDR3s)

    return CDR3s, idx_max, specs




# info: try to use the numba_test_2.py created sparse file to make a network

import numpy as np
#import networkx as nx
import matplotlib.pyplot as plt
import time

from graph_tool.all import *
from . import graphics

from matplotlib import rcParams

import pandas as pd 

import seaborn as sns
import numpy as np

import sys 
import argparse

from . import pathogen_analysis as pa
from . import load_net
from . import net_lib as nl
#import graph_support as gs
from . import spec as s
"""
def plot(xs, ys, log=True, name=None):
    ""
    Info: plot the array ys over the array xs with 
        some standard settings and save it
    Args: xs: array of x-values
        ys: array of y-values
        log: bool: True: log-log-plot; False: no log-plot
        name: name of the file to save to
    Returns: -
    ""
    
    fig_size, params = graphics.design(factor=1)
    rcParams.update({'figure.figsize': fig_size})

    fig_1, ax_1 = plt.subplots()

    ax_1.plot(xs, ys, '.')
    if log == True:
        ax_1.set_xscale('log')
        ax_1.set_yscale('log')
    if name!=None:
        fig_1.tight_layout()
        fig_1.savefig(name, dpi=300, bbox_inches="tight")
    plt.show()
"""
def load_idx(name):
    """
    Info: 
    Args: 
    Returns: 
    """
    
    with open(name, "r") as f:
        vals = np.loadtxt(f)
        idx_max = int(vals-1)
    return idx_max

def spec_analysis():
    """
    Info: correlate the specificity values with the values
    Args: -
    Returns: -
    """
    # TODO: check if the values in the dataset really denote the specificity

    data = load_net.convert_edges(name="data/spec_1.txt", len_x=1, single_sidelength=4*10**3)
    g = load_net.assemble_graph(data)
    CDR3s, idx_max, specs = s.specificity()
    
    xs = list()
    ys = list()
    for v, i in zip(g.vertices(), range(len(list(g.vertices())))):
        y = g.get_total_degrees([v])
        x = specs[i]
        ys.append(y)
        xs.append(x)
    plt.plot(xs, ys, '.')
    plt.show()

def main(): 
    """
    Info: Do all the graph analysis
    Args: -
    Returns: -
    """

    print("\n pathogens ...")
    # info: pathogens
    
    parser = argparse.ArgumentParser()
    parser.add_argument("--len_x", help="")
    parser.add_argument("--single_sidelength", help="")
    
    args = parser.parse_args()
    parser_len_x = int(args.len_x) if not args.len_x == None else 1#50
    parser_single_sidelength = int(args.single_sidelength) if not args.single_sidelength == None else 4*10**3
    
    pathogens = ["Influenza", "HIV", "Alzheimer", "Parkinson", "Tuberculosis", "COVID-19"]
    cols = {"Influenza": (1.0, 0.0, 0.0, 1.0), 
            "HIV": (0.0, 1.0, 0.0, 1.0), 
            "Alzheimer": (0.0, 0.0, 1.0, 1.0),
            "Parkinson": (0.0, 0.0, 0.0, 1.0), 
            "Tuberculosis": (0.5, 0.5, 0.5, 1.0), 
            "COVID-19": (0.5, 0.5, 0.0, 1.0)}
    ress = {}
    
    ress_pathogen = list()
    ress_val = list()
       
    fig, ax = plt.subplots(nrows=2, ncols=2)
    ax_00 = ax[0][0]
    ax_01 = ax[0][1]
    ax_10 = ax[1][0]
    ax_11 = ax[1][1]
    """
    # info:  normal analysis
    i = 1
    name = "data/sparse_" + str(i)+ ".txt"
    name_params = "data/sparse_" + str(i) + ".txt"
    print("\n normal: name: ", name, "; name_params: ", name_params)
    data = load_net.convert_edges(name=name, len_x=parser_len_x, single_sidelength=parser_single_sidelength)
    g = load_net.assemble_graph(data)
    nl.net_analysis(g)

    # info: pathogen analysis
    for pathogen in pathogens:
        dic_total = {}
        dic_total["betweenness_dist"] = {}
        dic_total["degree_dist"] = {}
        
        samples = 1
        for i in range(samples):
            #name = "data/" + pathogen + "_small.txt"
            name = "data/" + pathogen + "_small_" + str(i)+ ".txt"
            name_params = "data/" + pathogen + "_params_" + str(i) + ".txt"
            print("\n name: ", name)
            print("\n name_params: ", name_params)
            data = load_net.convert_edges(name=name, len_x=parser_len_x, single_sidelength=parser_single_sidelength)
            g = load_net.assemble_graph(data)
            idx_max = load_idx(name_params)
            print("\n loaded idx_max: ", idx_max)
            res_raw = pa.pathogen_analysis(g, idx_max)
            #TODO: load idx_max value
            ""betw
            res = res_raw["betweenness"][:idx_max]
            print("\n pathogen: ", pathogen, "; res: ", res)
            for i in range(len(res)):
                ress_pathogen.append(pathogen)
                ress_val.append(res[i])
            
            dic = res_raw["betweenness_dist"]
            keys = list(dic.keys())
            vals = list(dic.values())
            
            for j in range(len(vals)):
                try: 
                    dic_tota["betweenness_dist"][keys[j]].append(vals[j])
                except: 
                    dic_total["betweenness_dist"][keys[j]] = list()
                    dic_total["betweenness_dist"][keys[j]].append(vals[j])
            ""
            dic = res_raw["degree_dist"]
            keys = list(dic.keys())
            vals = list(dic.values())

            for j in range(len(vals)):
                try:
                    dic_total["degree_dist"][keys[j]].append(vals[j])
                except:
                    dic_total["degree_dist"][keys[j]] = list()
                    dic_total["degree_dist"][keys[j]].append(vals[j])
        ""betw
        for key in list(dic_total["betweenness_dist"].keys()):
            dic_total["betweenness_dist"][key] = np.mean(dic_total["betweenness_dist"][key])
        ""
        for key in list(dic_total["degree_dist"].keys()):
            dic_total["degree_dist"][key] = np.mean(dic_total["degree_dist"][key])
        keys = list(dic_total["betweenness_dist"].keys())
        vals = list(dic_total["betweenness_dist"].values())
          
        ax_10.plot(keys, vals, '.', color=cols[pathogen])
            
        dic = res_raw["degree_dist"]
        keys = list(dic_total["degree_dist"].keys())
        vals = list(dic_total["degree_dist"].values())
        ax_11.plot(keys, vals, '.', color=cols[pathogen])
        
        ax_01.plot(res_raw["raw_degs"], color=cols[pathogen])

    ax_10.set_title("pathogens")
    #ax_10.set_xscale("log")
    #ax_10.set_yscale("log")
    ax_11.set_xscale("log")
    ax_11.set_yscale("log")
    #res[pathogen] = res["betweenness"] 
    plt.show()   
    #xs, ys = viruses(g)
    ress["pathogen"] = ress_pathogen
    ress["val"] = ress_val
    data = pd.DataFrame(ress)
    sns.stripplot(data=data, x="pathogen", y="val", ax=ax_00)
    """ 
    
    spec_analysis()
    #data = pd.DataFrame(ress)
    #ax_01.plot(ress)
    """
    g = assemble_graph("sparse_virus.txt")
    """
    # TODO: some virus graph analysis

if __name__ == "__main__":
    main()



import textdistance as td
from . import funcDictionary as dic
import numpy as np

import sys
import argparse

from . import plotData

def ref_levenshtein(parser_N, src_name, ld_max):
    """
    Info: calculate the adjacency matrix with the normal textdistance
        Levenshtein distance to verify if our algorithm is correct included
    Args:  ii
    Returns: -
    """

    plotData.clusterMinLen = 1
    plotData.N = parser_N#10**6#4*10**3
    plotData.min_ldVal = -1
    plotData.maxVal = 3
    plotData.max_ldVal = plotData.maxVal
    gpu_l = 8000 #5000
    step = 0
    _, _, seq, _, _ = dic.loadSequence(step, plotData, isExtractNum=False, src=src_name)

    a = np.zeros([len(seq), len(seq)])

    for i in range(len(seq)):
        for j in range(len(seq)):
            dist = td.levenshtein(seq[i], seq[j])
            a[i][j] = 1 if dist < ld_max else 0

    name = "two_number_indices_ref.txt"
    with open(name, 'w') as f:
        np.savetxt(f, a, fmt='%i')
    print("\n saved under: ", name)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--N', help="")
    parser.add_argument('--src', help="")
    parser.add_argument('--ld_max', help="<ld_max = 1 else 0")
    args = parser.parse_args()

    parser_N = int(args.N)
    src_name = args.src
    ld_max = int(args.ld_max) if not args.ld_max == None else 2
    ref_levenshtein(parser_N, src_name, ld_max)
